/* morla - Copyright (C) 2005-2006 bakunin - Andrea Marchesini 
 *                                     <bakunin@somasuite.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#define HELP_TIMEOUT	10
#define HELP_MAX_LIST	10

#include "editor.h"

struct help_t
{
  gchar *input;
  gchar *output;

  gint flag;
};

gchar *help_url = NULL;

static GList *help_back_list;
static GList *help_forward_list;

static GtkWidget *help_back_widget;
static GtkWidget *help_forward_widget;
static GtkWidget *help_index_widget;

static GtkWidget *help_pb;
static GtkWidget *help_window;
static HtmlDocument *help_doc;

static gchar *help_uri;

static void help_page (gchar * page, gboolean rec);

/* HELP FUNCTIONS ***********************************************************/
static void
help_back_forward (void)
{
  gtk_widget_set_sensitive (help_back_widget, help_back_list ? TRUE : FALSE);
  gtk_widget_set_sensitive (help_forward_widget,
			    help_forward_list ? TRUE : FALSE);
}

static void
help_back (GtkWidget * w, gpointer dummy)
{
  gchar *uri;

  if (!help_back_list)
    return;

  if (help_uri)
    {
      help_forward_list = g_list_prepend (help_forward_list, help_uri);

      while (g_list_length (help_forward_list) > HELP_MAX_LIST)
	{
	  GList *list = g_list_last (help_forward_list);

	  g_free (list->data);
	  help_forward_list = g_list_remove (help_forward_list, list->data);
	}

      help_uri = NULL;
    }

  uri = help_back_list->data;
  help_back_list = g_list_remove (help_back_list, help_back_list->data);
  help_page (uri, FALSE);
  g_free (uri);

  help_back_forward ();
}

static void
help_forward (GtkWidget * w, gpointer dummy)
{
  gchar *uri;

  if (!help_forward_list)
    return;

  if (help_uri)
    {
      help_back_list = g_list_prepend (help_back_list, help_uri);

      while (g_list_length (help_back_list) > HELP_MAX_LIST)
	{
	  GList *list = g_list_last (help_back_list);

	  g_free (list->data);
	  help_back_list = g_list_remove (help_back_list, list->data);
	}

      help_uri = NULL;
    }

  uri = help_forward_list->data;
  help_forward_list =
    g_list_remove (help_forward_list, help_forward_list->data);
  help_page (uri, FALSE);
  g_free (uri);

  help_back_forward ();
}

static void
help_link (HtmlDocument * doc, const gchar * uri, gpointer dummy)
{
  if (strncmp (uri, "morla://", 8))
    browser ((gchar *) uri);

  else
    {
      gchar *str = NULL;

      if (help_uri)
	{
	  help_back_list = g_list_prepend (help_back_list, help_uri);
	  help_uri = NULL;
	}

      if (strlen (uri) > 8)
	str = g_strdup_printf ("%s/%s", help_url, uri + 8);

      help_page (str, FALSE);
      g_free (str);
    }
}

static void
help_null (void *fl, const char *str, ...)
{
  /* void */
}

struct help_download_t
{
  gchar *mm;
  gint size;
};

static size_t
help_memory (void *ptr, size_t size, size_t nmemb, void *data)
{
  gint realsize = size * nmemb;
  struct help_download_t *mem = (struct help_download_t *) data;

  if (!mem->mm)
    mem->mm = g_malloc (realsize + 1);
  else
    mem->mm = g_realloc (mem->mm, mem->size + realsize + 1);

  memcpy (mem->mm + mem->size, ptr, realsize);
  mem->size += realsize;
  mem->mm[mem->size] = 0;

  return realsize;
}

static gchar *
help_download_uri (gchar * uri)
{
  CURL *curl;
  struct help_download_t *chunk;
  gchar *ret;

  curl_global_init (CURL_GLOBAL_DEFAULT);

  if (!(curl = curl_easy_init ()))
    {
      curl_easy_cleanup (curl);
      return NULL;
    }

  chunk = g_malloc0 (sizeof (struct help_download_t));

  curl_easy_setopt (curl, CURLOPT_URL, uri);
  curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, help_memory);
  curl_easy_setopt (curl, CURLOPT_FOLLOWLOCATION, 1);
  curl_easy_setopt (curl, CURLOPT_FILE, (void *) chunk);

  curl_easy_setopt (curl, CURLOPT_TIMEOUT, HELP_TIMEOUT);

  if (curl_easy_perform (curl))
    {
      if (chunk->mm)
	g_free (chunk->mm);

      g_free (chunk);
      curl_easy_cleanup (curl);

      return NULL;
    }

  curl_easy_cleanup (curl);

  ret = chunk->mm;
  g_free (chunk);

  return ret;
}

static gchar *
help_download_file (gchar * file)
{
  gchar *ret;

  if (g_file_get_contents (file, &ret, NULL, NULL) == FALSE)
    return NULL;

  return ret;
}

static gchar *
help_xslt (gchar * memory)
{
  xsltStylesheetPtr cur = NULL;
  xmlDocPtr doc, res, ss_doc;
  xmlChar *ret;
  int len;
  gchar *output;
  gchar *ss;

  xmlSubstituteEntitiesDefault (1);
  xmlSetGenericErrorFunc (xmlGenericErrorContext, help_null);

  if (!strncmp (help_url, "http://", 7))
    {
      gchar *s = g_strdup_printf ("%s/xslt.xslt", help_url);
      ss = help_download_uri (s);
      g_free (s);
    }

  else
    {
      gchar *s = g_strdup_printf ("%s/xslt.xslt", help_url);
      ss = help_download_file (s);
      g_free (s);
    }

  if (!ss)
    return NULL;

  if (!(doc = xmlParseMemory (memory, strlen (memory))))
    return NULL;

  if (!(ss_doc = xmlParseMemory (ss, strlen (ss))))
    {
      xmlFreeDoc (doc);
      return NULL;
    }

  if (!(cur = xsltParseStylesheetDoc (ss_doc)))
    {
      xmlFreeDoc (doc);
      g_free (ss);
      return NULL;
    }

  g_free (ss);

  if (!(res = xsltApplyStylesheet (cur, doc, NULL)))
    {
      xsltFreeStylesheet (cur);
      xmlFreeDoc (doc);
      return NULL;
    }

  if (xsltSaveResultToString (&ret, &len, res, cur))
    {
      xsltFreeStylesheet (cur);
      xmlFreeDoc (doc);
      xmlFreeDoc (res);
      return NULL;
    }

  xsltFreeStylesheet (cur);
  xmlFreeDoc (doc);
  xmlFreeDoc (res);

  xsltCleanupGlobals ();
  xmlCleanupParser ();

  output = g_malloc (sizeof (gchar) * (len + 1));
  strncpy (output, (gchar *) ret, len);
  output[len] = 0;

  g_free (ret);

  return output;
}

static void *
thread (void *dummy)
{
  struct help_t *data = (struct help_t *) dummy;
  gchar *mm;

  if (!strncmp (data->input, "http://", 7))
    mm = help_download_uri (data->input);
  else
    mm = help_download_file (data->input);

  if (!mm)
    {
      data->flag = 1;
      g_thread_exit (NULL);
      return NULL;
    }

  data->output = help_xslt (mm);
  g_free (mm);

  data->flag = 1;
  g_thread_exit (NULL);
  return NULL;
}

static void
help_page (gchar * page, gboolean rec)
{
  GThread *th;
  struct help_t data;

  gtk_widget_set_sensitive (help_index_widget, FALSE);
  gtk_widget_set_sensitive (help_back_widget, FALSE);
  gtk_widget_set_sensitive (help_forward_widget, FALSE);

  if (!page)
    page = g_strdup_printf ("%s/index.xml", help_url);
  else
    page = g_strdup (page);

  data.input = page;
  data.output = NULL;
  data.flag = 0;

  th = g_thread_create ((GThreadFunc) thread, &data, FALSE, NULL);

  while (!data.flag)
    {
      gtk_progress_bar_pulse (GTK_PROGRESS_BAR (help_pb));

      while (gtk_events_pending () && !data.flag)
	gtk_main_iteration ();

      g_usleep (50000);
    }

  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (help_pb), 0);

  html_document_clear (help_doc);
  html_document_open_stream (help_doc, "text/html");

  if (!data.output)
    {
      if (rec == FALSE)
	{
	  gchar *s = g_strdup_printf ("%s/error.xml", help_url);
	  help_page (s, TRUE);
	  g_free (s);
	}

      else
	{
	  gchar *tmp;

	  tmp =
	    g_strdup_printf
	    ("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n"
	     "    <head>\n" "      <title>%s</title>\n"
	     "      <link rel=\"meta\" href=\"http://www.autistici.org/bakunin/morla/doap.rdf\" title=\"rdf\"/>\n"
	     "      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>\n"
	     "      <style type=\"text/css\">\n" "        #title {\n"
	     "	  background: #CCC none;\n"
	     "	  border: 1px solid #000;\n" "	  padding: 2px;\n"
	     "	}\n" "\n" "        #subtitle {\n"
	     "	  padding: 2px 2px 10px 2px;\n" "	}\n"
	     "\n" "        #content {\n"
	     "	  border-top: 1px dashed #000;\n"
	     "	  border-bottom: 1px dashed #000;\n"
	     "	  padding: 0px;\n" "	}\n"
	     "\n" "        #end {\n"
	     "	  padding: 10px;\n"
	     "	  text-align: right;\n"
	     "	}\n" "\n"
	     "        #sectitle {\n"
	     "	  padding: 5px 2px 2px 2px;\n"
	     "	  font-size: 14px;\n"
	     "	}\n"
	     "\n"
	     "        a {\n"
	     "          text-decoration: none;\n"
	     "        }\n"
	     "\n"
	     "        a:hover {\n"
	     "          text-decoration: underline;\n"
	     "	}\n"
	     "      </style>\n"
	     "    </head>\n"
	     "    \n"
	     "    <body>\n"
	     "      <div id=\"title\"><h1>%s</h1></div>\n"
	     "      <div id=\"content\">%s</div>\n"
	     "    </body>\n"
	     "  </html>\n",
	     _
	     ("Error!"),
	     _
	     ("Error!"),
	     _
	     ("The error file does not exist, or your computer has some problem with the network!"));

	  html_document_write_stream (help_doc, tmp, strlen (tmp));

	  g_free (tmp);
	}
    }

  else
    {
      html_document_write_stream (help_doc, data.output,
				  strlen (data.output));
      g_free (data.output);

      if (help_uri)
	g_free (help_uri);

      help_uri = g_strdup (page);
    }

  gtk_widget_set_sensitive (help_index_widget, TRUE);
  help_back_forward ();

  g_free (data.input);
}

static void
help_index (GtkWidget * w, gpointer dummy)
{
  help_page (NULL, FALSE);
}

static void
help_destroy (GtkWidget * w, gpointer dummy)
{
  if (help_back_list)
    {
      g_list_foreach (help_back_list, (GFunc) g_free, NULL);
      g_list_free (help_back_list);
      help_back_list = NULL;
    }

  if (help_forward_list)
    {
      g_list_foreach (help_forward_list, (GFunc) g_free, NULL);
      g_list_free (help_forward_list);
      help_forward_list = NULL;
    }

  if (help_uri)
    {
      g_free (help_uri);
      help_uri = NULL;
    }

  help_doc = NULL;
}

void
help (void)
{
  GtkWidget *window;
  GtkWidget *frame;
  GtkWidget *sw;
  GtkWidget *vbox, *box, *bbox;
  GtkWidget *button;
  GtkWidget *label;
  GtkWidget *image;
  GtkWidget *html;

  gchar s[1024];
  gint x, y;

  if (help_doc)
    {
      gtk_window_present (GTK_WINDOW (help_window));
      return;
    }

  g_snprintf (s, sizeof (s), "%s %s - %s", PACKAGE, VERSION, _("Help"));

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), s);
  g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (help_destroy),
		    NULL);
  help_window = window;

  x = gdk_screen_width () * 1 / 4;

  if (x < 250)
    x = 250;

  y = gdk_screen_width () * 1 / 2;

  if (y < 400)
    y = 400;

  gtk_window_resize (GTK_WINDOW (window), x, y);

  vbox = gtk_vbox_new (0, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  box = gtk_hbox_new (0, 0);
  gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);

  button = gtk_button_new ();
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  help_back_widget = button;
  gtk_widget_set_sensitive (button, FALSE);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (help_back),
		    NULL);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);

  bbox = gtk_vbox_new (0, 0);
  gtk_container_add (GTK_CONTAINER (button), bbox);

  image = gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_BUTTON);
  gtk_box_pack_start (GTK_BOX (bbox), image, FALSE, FALSE, 0);

  label = gtk_label_new (_("Back"));
  gtk_box_pack_start (GTK_BOX (bbox), label, TRUE, TRUE, 0);

  button = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);

  button = gtk_button_new ();
  help_index_widget = button;
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (help_index),
		    NULL);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);

  bbox = gtk_vbox_new (0, 0);
  gtk_container_add (GTK_CONTAINER (button), bbox);

  image = gtk_image_new_from_stock ("gtk-index", GTK_ICON_SIZE_BUTTON);
  gtk_box_pack_start (GTK_BOX (bbox), image, FALSE, FALSE, 0);

  label = gtk_label_new (_("Index"));
  gtk_box_pack_start (GTK_BOX (bbox), label, TRUE, TRUE, 0);

  button = gtk_vseparator_new ();
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);

  button = gtk_button_new ();
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  help_forward_widget = button;
  gtk_widget_set_sensitive (button, FALSE);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (help_forward),
		    NULL);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);

  bbox = gtk_vbox_new (0, 0);
  gtk_container_add (GTK_CONTAINER (button), bbox);

  image = gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_BUTTON);
  gtk_box_pack_start (GTK_BOX (bbox), image, FALSE, FALSE, 0);

  label = gtk_label_new (_("Forward"));
  gtk_box_pack_start (GTK_BOX (bbox), label, TRUE, TRUE, 0);

  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (frame), sw);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  help_doc = html_document_new ();
  g_signal_connect (G_OBJECT (help_doc), "link_clicked",
		    G_CALLBACK (help_link), NULL);

  html_document_open_stream (help_doc, "text/html");

  html = html_view_new ();
  gtk_container_add (GTK_CONTAINER (sw), html);
  html_view_set_document (HTML_VIEW (html), help_doc);

  help_pb = gtk_progress_bar_new ();
  gtk_box_pack_start (GTK_BOX (vbox), help_pb, FALSE, FALSE, 0);

  gtk_widget_show_all (window);

  help_page (NULL, FALSE);
}

/* EOF */
